%% ---------------- Problem 1
clear;
% First create the data matrix, row 1 is gene1 and row 2 is gene2
X=[3 2 4 0 6 3 1 5 -1 7; 1 3 -1 7 -5 1 0 2 -1 3];
N=size(X,2); % we will use this over and over again, so let's store it now.
             % N represents the number of samples (points) in the data
% ---- part a: plot gene2 expression vs gene1 expression
figure(1); clf;
plot(X(1,:),X(2,:),'.k','markersize',20)
ylabel('gene2','fontsize',16);
xlabel('gene1','fontsize',16)
axis('square',8*[-1 1 -1 1])
% ---- part b: find the mean values for each gene
Xmu=mean(X,2); % the 2 tells matlab that you want the mean across the columns for each row.
% ---- part c: get variance
Xvar=var(X,1,2); % the 2 says across columns again, the 1 says to use normalization by N and not N-1
% ---- part d
% -- i: mean subtract
Xtilde=X-repmat(Xmu,1,N); % avoids for-loop by replicating the vector mu into a matrix
% -- ii: covariance matrix of Xtilde
C=Xtilde*Xtilde'/N;
% -- iii: eigenvalues of C
[V,D]=eig(C);
[V,eigvals]=sort_eigenvectors(V,D); % sort_eigenvectors is a function I wrote and is
                                    % attached with the solutions.  It will handle
                                    % the sorting of eigenvectors and eigenvalues
                                    % into a single step.
% -- iv: total fraction of variance of first PC
frac_var=eigvals/sum(eigvals); % the fraction of the first PC is in the first element
                               % of frac_var.
% part d -- v: plot eigenvectors with data
% we can do this in a brand new plot, but let's do it on the same plot from part a
figure(1); % brings figure 1 on top (in case there are other figures hanging out)
line([0 3*V(1,1)]+Xmu(1),[0 3*V(2,1)]+Xmu(2),'color','g','linewidth',2);
line([0 3*V(1,2)]+Xmu(1),[0 3*V(2,2)]+Xmu(2),'color','b','linewidth',2);
% ---- part e: reexpress the data onto pca space
Y=V'*Xtilde; % don't forget to use mean subtracted data!
% ---- part f: plot time series'
figure(2); clf;
fig_pos2=get(2,'position'); % this will return the screen position and size of figure 2 as a vector
set(2,'position',[fig_pos2(1:2) 600 300]); % this resizes figure 2, but leaves it in 
                                           % the same location.  The figure will now
                                           % be 600 pixels wide and 300 pixels high.
ha(1)=subplot(2,2,1); % creates a multipaneledd figure with 2 rows and 2 cols. The 
                      % 3rd input tells Matlab that I want to work on the first 
                      % (row 1, col 1) right now, putting a different number will let
                      % me work on a different one.
                      % The output ha(1) is not necessary, but it will make changing 
                      % axes properties simpler later on.
h(1)=plot(0:10:40,Y(1,1:5)); % h is a handle to each plot, we will use it below
title('PC1 (heat shock)')
ha(2)=subplot(2,2,2); % the second panel (row 1, col 2)
h(2)=plot(0:4,Y(1,6:10));
title('PC2 (glucose to glycerol)')
ha(3)=subplot(2,2,3); % the third panel (row 2, col 1)
h(3)=plot(0:10:40,Y(2,1:5));
title('PC2 (heat shock)')
xlabel('time (mins)');
ha(4)=subplot(2,2,4); % the fourth panel (row 2, col 2)
h(4)=plot(0:4,Y(2,6:10));
title('PC2 (glucose to glycerol)')
xlabel('time (hours)');
% set some line properties
set(h,'color','k','linewidth',3) % notice how I can send the entire vector of handles 
                                 % to set.  This greatly simplifies changing a bunch
                                 % of line properties all at once.
% set axis properties
set(ha,'ylim',[-10 10]) % Again, notice how I can all of the axes simultaneously.
                        % In this case, I'm changing all of the y limits to run from
                        % -10 to 10 so that all of the data can be viewed at the same
                        % scale.
% ---- part g: intepretation
% even though both genes express during both heat shock and the shift from glucose to
% glycerol, they seem to reflect an underlying process that responds to heat shock
% and glycerol separately.

%% ---------------- Problem 2
clear;
X=[12.1 6.6 11.5 8.7 15.4 6.9 8.8 10.1 19.1 15.6 22.2 10.2 15.7 10.0 7.2 11.8 11.3 12.5 11.4 12.9; ...
   6.9  7.5  6.1 8.1  9.0 6.0 8.5  8.3 11.2  7.6  9.3  7.5  9.2  6.4 6.3  7.7  8.3  7.5  9.5  9.3];
N=size(X,2);
% ---- part a: covariance without centering
Cdumb=X*X'/N;
% ---- part b: diagonalize the covariance matrix
[Vdumb,Ddumb]=eig(Cdumb);
[Vdumb,eigvals_dumb]=sort_eigenvectors(Vdumb,Ddumb);
% ---- part c: transform into PCA space
Ydumb=Vdumb'*X;
figure(11); clf;
plot(Ydumb(1,:),Ydumb(2,:),'.k','markersize',20)
axis('square',30*[-1 1 -1 1])
% ---- part d: now mean subtract
% we are going to do most of the parts in a single step using a function I wrote 
% called pca.  It will accept the uncentered data, and return the data in the pca 
% space as well as the centered data in the original space.
[V,Y,eigvals,C,Xtilde]=pca(X);
figure(12); clf
plot(Y(1,:),Y(2,:),'.k','markersize',20)
axis('square',15*[-1 1 -1 1])

%% ---------------- Problem 3
clear
load pca_homeworkdata
% the data is already mean subtracted, so we don't need to do it here.  However, you
% should remember to always make sure the data is centered before proceeding with pca
% ---- part a: 
colormatrix=jet(10);
figure(101); clf;
for k=1:size(Y,1)
    line(time,Y(k,:),'color',colormatrix(k,:));
end
xlabel('time (arbitrary units)')
ylabel('y (arbitrary units)')
% ---- part b:
figure(102); clf;
fig_pos=get(102,'position'); % this will return the screen position and size of figure 102 as a vector
set(102,'position',[fig_pos(1:2) 700 250]);
pairs=[4 6; 2 3; 5 7]; % hand pick dimensions for plot
for k=1:3
    subplot(1,3,k);
    plot(Y(pairs(k,1),:),Y(pairs(k,2),:),'.k','markersize',14);
    axis('square',3*[-1 1 -1 1]);
    xlabel(['dim ' num2str(pairs(k,1))]); % num2str converts numbers into strings
    ylabel(['dim ' num2str(pairs(k,2))]);
end
% ---- part c: covariance matrix
figure(103); clf;
C=Y*Y'/size(Y,2);
imagesc(C); colorbar;
% ---- part d: pca the data
[V,X,eigvals]=pca(Y); % using the good one shot function again
% -- PC's vs time
figure(104); clf;
for k=1:size(X,1)
    line(time,X(k,:),'color',colormatrix(k,:));
end
xlabel('time (arbitrary units)')
ylabel('x (arbitrary units)')
% -- PC's vs PC's
figure(105); clf;
fig_pos=get(105,'position'); % this will return the screen position and size of figure 102 as a vector
set(105,'position',[fig_pos(1:2) 700 250]);
pairs=[1 2; 2 3; 3 4]; % hand pick dimensions for plot
for k=1:3
    subplot(1,3,k);
    plot(X(pairs(k,1),:),X(pairs(k,2),:),'.k','markersize',14);
    axis('square',3*[-1 1 -1 1]);
    xlabel(['dim ' num2str(pairs(k,1))]); % num2str converts numbers into strings
    ylabel(['dim ' num2str(pairs(k,2))]);
end
% -- covariance matrix
D=diag(eigvals);
figure(106); clf;
imagesc(D); colorbar;
% In the PCA space, the data is really only varying along three dimensions.  The rest
% of the PC's look like noise.  The covariance matrix is now only non-zero along the
% diagonal.
% ---- part e: scree plot
figure(107);
plot(eigvals,'-k.','linewidth',3,'markersize',24);
ylabel('variance')
xlabel('principal component number')
% ---- part f: cumulative scree plot
figure(108);
plot(100*cumsum(eigvals)/sum(eigvals),'-k.','linewidth',3,'markersize',24);
% cumsum returns the cumulative sum of the values for each element, that is calling
% cumsum([1 2 3 4 5]) would return [1 3 6 10 15].  We have to divide this by the
% total variance, hence the sum(eigvals) term in the denominator.
ylabel('cumulative pct variance')
xlabel('principal component number')
% this plot is useful if you wanted to say cutoff the dimensionality of your system
% at some (arbitrary) threshold, say 95%.





